# 94. Binary Tree Inorder Traversal

Given a binary tree, return theinordertraversal of its nodes' values.

For example:  
Given binary tree`[1,null,2,3]`,

```
   1
    \
     2
    /
   3
```

return`[1,3,2]`.

**Note:**Recursive solution is trivial, could you do it iteratively?

**Trivial Solution: According to the definition of Inorder  \(O\(n\) time and O\(n\) space, for the function call stack\)**

```cpp
class Solution {
    vector<int> answer;
public:
    vector<int> inorderTraversal(TreeNode* root) {

        inorderTraversalHelper(root);
        return answer;        
    }

    void inorderTraversalHelper(TreeNode * root){
        if(!root) return;
        inorderTraversalHelper(root->left);
        answer.push_back(root->val);
        inorderTraversalHelper(root->right);
    }
};
```

**Thoughts**

1. Using Stack to keep track path
2. \*Morris traversal

**Code** **Using Stack: \(O\(n\) time and O\(n\) space\)**

```cpp
/**
 * Definition for a binary tree node.
 * struct TreeNode {
 *     int val;
 *     TreeNode *left;
 *     TreeNode *right;
 *     TreeNode(int x) : val(x), left(NULL), right(NULL) {}
 * };
 */
class Solution {
    stack<TreeNode *> path;
    vector<int> answer;
public:
    vector<int> inorderTraversal(TreeNode* root) {

        traverse(root);
        return answer;
    }

    void traverse(TreeNode* cur){
        while(cur || !path.empty()){
            // first to the left
            while(cur){
                path.push(cur);
                cur = cur -> left;
            }
            // left done , do cur
            cur = path.top(); path.pop();
            answer.push_back(cur->val);
            // exploring right
            cur = cur->right;
        }
    }
};
```

**Code with **[**Morris Traversal**](https://stackoverflow.com/questions/5502916/explain-morris-inorder-tree-traversal-without-using-stacks-or-recursion)**: O\(n\) time and O\(1\) space \(two assisting pointers\)!**

psudo-code \(Thanks [monkeykingyan](https://leetcode.com/monkeykingyan)'s [solution](https://techgeekyan.blogspot.ca/2017/08/leetcode-solution-94-binary-tree.html)\)

```
Step1. Initialize current as root
Step2. While current is not NULL
  If current does not have left child
  a. Add current’s value
  b. Go to the right, i.e., current = current.right
 Else
  a. In current's left subtree, make current the right child of the rightmost node
  b. Go to this left child, i.e., current = current.left
```

**Code**

```cpp
/**
 * Definition for a binary tree node.
 * struct TreeNode {
 *     int val;
 *     TreeNode *left;
 *     TreeNode *right;
 *     TreeNode(int x) : val(x), left(NULL), right(NULL) {}
 * };
 */
class Solution {
public:
    vector<int> inorderTraversal(TreeNode* root) {
        vector<int> answer;
        TreeNode *cur = root; 
        while(cur){

            if(cur->left){
                TreeNode* pre = cur -> left;
                while(pre -> right) pre = pre -> right;
                pre->right = cur;
                TreeNode * temp = cur;
                cur = cur -> left;
                temp -> left = NULL;
            }
            else{
                answer.push_back(cur->val);
                cur = cur -> right;
            }
        }
         return answer;
    }
};
```

[Talonj @ Stackoverflow](https://stackoverflow.com/questions/5502916/explain-morris-inorder-tree-traversal-without-using-stacks-or-recursion) explaination:

Special Thanks [jianchaolifighter](https://leetcode.com/jianchaolifighter) for the [reference](https://leetcode.com/problems/binary-tree-inorder-traversal/discuss/31231)

